require 'uu/os/cmd/command_controller'
require 'uu/os/cmd/execution_context'
require 'uu/os/cmd/callback_execution_context'
require 'uu/os/cmd/command_error'
require 'uu/os/security/session'

module UU
  module OS
    module CMD

      # Ancestor for development of asynchronous commands. See documentation
      # of {UU::OS::CMD::CommandController CommandController} for basic
      # information.
      #
      # Method {#on_exec on_exec} is always invoked synchronously. It should contain
      # only necessary validation of execution preconditions. If validation
      # passes, {UU::OS::CMD::ExecutionContext#invoke_async invoke_async}
      # method should be invoked on execution context which handles asynchronous
      # invocation of {#on_exec_async on_exec_async} method.
      class AsyncCommandController < UU::OS::CMD::CommandController

        # Method containing necessary validations (and other preparation tasks) to be
        # done before asynchronous part of command is invoked.
        # @param [UU::OS::CMD::ExecutionContext] ctx Instance of execution context.
        # @return [Object] Value obtained by {UU::OS::CMD::ExecutionContext#invoke_async
        #    invoke_async} method representing asynchronous task to be returned to client (and/or
        #    used in {UU::OS::CMD::ExecutionContext#invoke_async_status invoke_async_status}
        #    method).
        def on_exec(ctx)
          raise UU::OS::CMD::CommandError.new(UU::OS::IPC::ErrorKind::SERVER, UU::OS::IPC::ErrorCodes::NOT_IMPLEMENTED,
            "Method on_exec is not implemented on #{self.class.name}.")
        end

        # Main action method of asynchronous controller. This method is called
        # asynchronously once {UU::OS::CMD::ExecutionContext#invoke_async
        # invoke_async} method on execution context is invoked.
        # @param [UU::OS::CMD::ExecutionContext] ctx Instance of execution
        #    context.
        # @return [URI, String] Result of asynchronous method. Currently only
        #    URI is supported as return value.
        def on_exec_async(ctx)
          raise UU::OS::CMD::CommandError.new(UU::OS::IPC::ErrorKind::SERVER, UU::OS::IPC::ErrorCodes::NOT_IMPLEMENTED,
            "Method on_exec_async is not implemented on #{self.class.name}.")
        end

        # Method provides actual status of asynchronous task. This can be typically done
        # by invocation of {UU::OS::CMD::ExecutionContext#invoke_async_status} method on execution 
        # context and returning its result.
        #  
        # @param [UU::OS::CMD::ExecutionContext] ctx Instance of execution
        #    context.
        # @return [Object] Object representing status of asynchronous task.
        def on_exec_async_status(ctx)
          return ctx.invoke_async_status(ctx.parameters[:taskUri])
        end

        # Adds method for invocation of asynchronous part to context.
        # @private
        def get_context(request, uu_uri, parameters) # :nodoc:
          data = {}
          data[:request] = request
          data[:uu_uri] = uu_uri
          data[:parameters] = parameters
          data[:session] = UU::OS::Security::Session.current_session
          ctx = parameters.delete(context_parameter_name)
          if !ctx.nil? && ctx.has_key?(:result)
            data[:result] = ctx[:result]
            return UU::OS::CMD::CallbackExecutionContext.new(data)
          else
            data[:callback] = ctx[:callback] if !ctx.nil?
            return UU::OS::CMD::ExecutionContext.new(data)
          end
        end

      end

    end
  end
end
