require 'uu/os/cmd/execution_context'
require 'uu/os/cmd/command_error'

module UU
  module OS
    module CMD

      # Callback execution context.
      class CallbackExecutionContext < UU::OS::CMD::ExecutionContext

        # State of asynchronous command for which was callback invoked.
        # @return [String] Asynchronous command state (value from {UU::OS::CMD::AsyncCommandState}).
        attr_reader :state

        # Result of asynchronous command for which was callback invoked. May be +nil+ if asynchronous
        # command did not provide any result or invocation of asynchronous command failed.
        # @return [Object, nil] Asynchronous command result.
        attr_reader :result

        # Error raised from invocation of asynchronous command for which was callback invoked.
        # In case asynchronous command finished successfully, +exception+ should be +nil+.
        # @return [Exception, nil] Asynchronous command error.
        attr_reader :exception

        # Creates new instance of callback execution context.
        # @param [Hash] data Initialization data:
        # @option data [URI, String] :uu_uri URI of main object.
        # @option data [Hash] :parameters Use case parameters.
        # @option data [UU::OS::Security::Session] :session Instance of session object.
        # @option data [UU::OS::HTTP::HTTPRequest] :request Instance of request.
        # @option data [Hash] :result Callback result.
        # @private
        def initialize(data = {}) # :nodoc:
          super(data)
          return if data[:result].nil?
          @state = data[:result][:state]
          @result = data[:result][:result]
          ex = data[:result][:exception]
          if !ex.nil?
            @exception = UU::OS::CMD::CommandError.new(ex)
          end
        end

        # Prepares callback object to be used in {UU::OS::CMD::CommandClient}.
        # @param [Symbol, String] action Action to be invoked as callback handler.
        # @param [UU::OS::Security::Session] session Instance of session object to
        #    use for callback. If not set, callback will be invoked with the same
        #    session as asynchronous command for which the callback is defined.
        # @param [Hash] options Additional callback options.
        # @option options [Hash] :parameters Parameters to send to callback handler.
        # @return [Object] Callback object to be passed to {UU::OS::CMD::CommandClient#invoke
        #    CommandClient}.
        def prepare_callback(action, session = nil, options = {})
          raise 'Not supported for callbacks'
        end

      end

    end
  end
end
