require 'uri'
require 'uu/os/util/name_converter'
require 'uu/os/application/config'
require 'uu/os/cmd/command_error'
require 'uu/os/cmd/error_codes'

module UU
  module OS
    module CMD
      # Use case execution context.
      class ExecutionContext
        include UU::OS::Util::NameConverter

        # Default name of parameter in which are default application credentials, this credentials
        # are used for Message bus, Object Store etc.
        # @private
        APP_CREDENTIALS_PARAMETER_NAME = :app_credentials # :nodoc:
        private_constant :APP_CREDENTIALS_PARAMETER_NAME

        # URI of main object (object to be processed by invoked command).
        # @return [String] URI of main object.
        attr_reader :uu_uri

        # Use case parameters.
        # @return [Hash] Use case parameters.
        attr_reader :parameters

        # Session object.
        # @return [UU::OS::Security::Session] Session object.
        attr_reader :session

        # Instance of use case request. Note that request body
        # might not be available (once use case parameters are
        # parsed).
        # @return [UU::OS::CON::HTTP::HTTPRequest] Instance of request.
        # @private
        attr_reader :request # :nodoc:
        
        # Creates new instance of execution context.
        # @param [Hash] data Initialization data:
        # @option data [URI, String] :uu_uri URI of main object.
        # @option data [Hash] :parameters Use case parameters.
        # @option data [UU::OS::Security::Session] :session Instance of session object.
        # @option data [UU::OS::HTTP::HTTPRequest] :request Instance of request.
        # @option data [TrueClass, FalseClass] :forbid_async  If true, so asynchronous invocation is forbidden, 
        # it means that invocation of invoke_async and invoke_async_status ends up with exception.
        # @private
        def initialize(data = {}) # :nodoc:
          @uu_uri = data[:uu_uri]
          @parameters = data[:parameters]
          @session = data[:session]
          @request = data[:request]
          @callback = data[:callback]
          @forbid_async = data[:forbid_async]
        end

        # Prepares callback object to be used in {UU::OS::CMD::CommandClient}.
        # @param [Symbol, String] action Action to be invoked as callback handler.
        # @param [UU::OS::Security::Session] session Instance of session representing
        #    user to be used for callback invocation. If not set, callback will be
        #    invoked under the same user who invoked command in which callback is
        #    prepared (initial user).
        # @param [Hash] options Additional callback options.
        # @option options [Hash] :parameters Parameters to send to callback handler.
        # @return [Object] Callback object to be passed to {UU::OS::CMD::CommandClient#invoke
        #    CommandClient}.
        def prepare_callback(action, session = nil, options = {})
          raise ArgumentError, 'action is not set' if action.nil? || action.empty?
          session, options = nil, session if session.kind_of?(Hash)

          qualified_use_case_code = URI.split(request.url)[5]
          qualified_use_case_code.gsub!(/(^\/)|(\/$)/, '')
          parts = qualified_use_case_code.split('/')
          parts.delete_at(-1) # Remove original action
          app_url_code = parts.delete_at(0)
          cmd_url_path = parts.join('/')

          callback = {}
          callback[:appUrlCode] = app_url_code
          callback[:cmdUrlPath] = cmd_url_path
          callback[:action] = to_camel_case(action)
          callback[:mainObjectUri] = uu_uri
          ses = session || @session
          if ses.kind_of?Hash
            callback[:authToken] = ses[:accessToken]
          elsif ses.respond_to?(:access_token)
            callback[:authToken] = ses.access_token
          elsif ses.kind_of?(String)
            callback[:authToken] = ses
          end
          callback[:parameters] = options[:parameters] if !options.nil?
          return callback
        end

        # Method for invocation of asynchronous part of asynchronous command.
        # Asynchronous part is always invoked under the same user who invoked
        # synchronous part (initial user).
        # @param [URI, String] queue URI of queue for invocation of asynchronous
        #    commands.
        # @param [UU::OS::Security::Session] session Instance of session object
        #    representing user allowed to use given queue (user allowed to plan
        #    invocation of asynchronous part). If not set, session representing
        #    currently authenticated user will be used. This parameter does not
        #    affect user under which asynchronous part is invoked (it is always
        #    invoked under the same user who invoked synchronous part).
        # @param [Hash] options Additional invocation options.
        # @option options [Hash] :parameters Parameters to use for invocation of
        #    asynchronous part. If not set, the same parameters as for invocation of
        #    synchronous part will be used.
        # @return [Object] Object representing asynchronous task to be returned
        #    to client (and/or used in {#invoke_async_status invoke_async_status}
        #    method).
        def invoke_async(queue, session = nil, options = {})
          if @forbid_async
            raise UU::OS::CMD::CommandError.new(UU::OS::IPC::ErrorKind::SERVER, UU::OS::CMD::ErrorCodes::UNEXPECTED_PROBLEM,
              "Calling invoke_async is not allowed for synchronous uuCommand.")
          end
          session, options = nil, session if session.kind_of?(Hash)
          qualified_use_case_code = URI.split(request.url)[5]
          qualified_use_case_code.gsub!(/(^\/)|(\/$)/, '')
          parts = qualified_use_case_code.split('/')
          action = parts.delete_at(-1)
          app_url_code = parts.delete_at(0)
          cmd_url_path = parts.join('/')

          params = {}
          params[:queue_uri] = queue
          params[:app_url_code] = app_url_code
          params[:cmd_url_path] = cmd_url_path
          params[:action] = action
          params[:parameters] = options[:parameters] || parameters
          params[:callback] = @callback
          params[:session] = @session
          client = UU::OS::CMD::CommandExecutor.new(session || get_credentials)
          task_uri = client.invoke(uu_uri, params)
          return {future: true, taskUri: task_uri, appUrlCode: app_url_code, cmdUrlPath: cmd_url_path,
            action: action, uuUri: uu_uri}
        end

        # Method for checking actual status of asynchronous task.
        # @param [URI, String, Object] task_ref URI of asynchronous task or object
        #    returned by {#invoke_async invoke_async} method.
        # @param [UU::OS::Security::Session] session Instance of session object
        #    representing user allowed to use queue where the task is executed.
        #    If not set, session representing currently authenticated user will
        #    be used.
        # @return [Object] Object representing status of asynchronous task.
        def invoke_async_status(task_ref, session = nil)
          if @forbid_async
            raise UU::OS::CMD::CommandError.new(UU::OS::IPC::ErrorKind::SERVER, UU::OS::CMD::ErrorCodes::UNEXPECTED_PROBLEM,
              "Calling invoke_async_status is not allowed for synchronous uuCommand.")
          end
          task_uri = task_ref.kind_of?(Hash) ? task_ref[:taskUri] : task_ref
          client = UU::OS::CMD::CommandExecutor.new(session || get_credentials)
          return client.get_state(task_uri)
        end

        # Override inspect to hide internal structures.
        # @private
        def inspect # :nodoc:
          "#<#{self.class.name}:0x#{(self.object_id << 1).to_s(16)}>"
        end

        private
        
        # Method returns session created from default app config credentials if are set or current session
        #
        # @return [UU::OS::Security::Session]
        # @private
        def get_credentials # :nodoc:
          # try if credentials are set in app config
          credentials = UU::OS::Application::Config[APP_CREDENTIALS_PARAMETER_NAME]
          queue_order_session = @session.login(*credentials) if !credentials.nil? && !credentials.empty?
          # if there is no credentials in app config get current session
          queue_order_session = UU::OS::Security::Session.current_session unless queue_order_session
          return queue_order_session
        end

      end

    end
  end
end
